@@ -5,6 +5,7 @@ This file contains functions that might be generally useful.
 import contextlib
 import datetime
 import functools
+import hashlib
 import inspect
 import math
 import io
@@ -47,6 +48,8 @@ class _PRINT:
             self._printer(*args, **kwargs)
 
 
+prompt_for_input = input  # In Python 3, this does 'safe' input reading.
+
 PRINT = _PRINT()
 PRINT.__name__ = 'PRINT'
 
@@ -1151,14 +1154,28 @@ def file_contents(filename, binary=False):
         return fp.read()
 
 
-def camel_case_to_snake_case(s):
-    """ Converts CamelCase to snake_case """
-    return ''.join('_' + c.lower() if c.isupper() else c for c in s).lstrip('_')
+def camel_case_to_snake_case(s, separator='_'):
+    """
+    Converts CamelCase to snake_case.
+    With a separator argument (default '_'), use that character instead for snake_case.
+    e.g., with separator='-', you'll get snake-case.
+
+    :param s: a string to convert
+    :param separator: the snake-case separator character (default '_')
+    """
+    return ''.join(separator + c.lower() if c.isupper() else c for c in s).lstrip(separator)
 
 
-def snake_case_to_camel_case(s):
-    """ Converts snake_case to CamelCase - note that "our" CamelCase always capitalizes the first character. """
-    return s.title().replace('_', '')
+def snake_case_to_camel_case(s, separator='_'):
+    """
+    Converts snake_case to CamelCase. (Note that "our" CamelCase always capitalizes the first character.)
+    With a separator argument (default '_'), expect that character instead for snake_case.
+    e.g., with separator='-', you'll expect snake-case.
+
+    :param s: a string to convert
+    :param separator: the snake-case separator character (default '_')
+    """
+    return s.title().replace(separator, '')
 
 
 def capitalize1(s):
@@ -1169,6 +1186,25 @@ def capitalize1(s):
     return s[:1].upper() + s[1:]
 
 
+def string_list(s):
+    """
+    Turns a comma-separated list into an actual list, trimming whitespace and ignoring nulls.
+        >>> string_list('foo, bar,,baz ')
+        ['foo', 'bar', 'baz']
+    """
+
+    if not isinstance(s, str):
+        raise ValueError(f"Not a string: {s!r}")
+    return [p for p in [part.strip() for part in s.split(",")] if p]
+
+
+def string_md5(unicode_string):
+    """
+    Returns the md5 signature for the given u unicode string.
+    """
+    return hashlib.md5(unicode_string.encode('utf-8')).hexdigest()
+
+
 class CachedField:
     def __init__(self, name, update_function, timeout=600):
         """ Provides a named field that is cached for a certain period of time. The value is computed
@@ -1448,6 +1484,40 @@ def dict_zip(dict1, dict2):
     return res
 
 
-# Deprecated names, still supported for a while.
-HMS_TZ = REF_TZ
-hms_now = ref_now
+def json_leaf_subst(exp, substitutions):
+    """
+    Given an expression and some substitutions, substitutes all occurrences of the given substitutions.
+    For example:
+
+    >>> json_leaf_subst({'xxx': ['foo', 'bar', 'baz']}, {'foo': 'fu', 'bar': 'bah', 'xxx': 'yyy'})
+    {'yyy': ['fu', 'bah', 'baz']}
+
+    :param exp: a JSON expression, represented in Python as a string, a number, a list, or a dict
+    :param substitutions: a dictionary of replacements from keys to values
+    """
+    def do_subst(e):
+        return json_leaf_subst(e, substitutions)
+    if isinstance(exp, dict):
+        return {do_subst(k): do_subst(v) for k, v in exp.items()}
+    elif isinstance(exp, list):
+        return [do_subst(e) for e in exp]
+    elif exp in substitutions:  # Something atomic like a string or number
+        return substitutions[exp]
+    return exp
+
+
+class NamedObject(object):
+
+    def __init__(self, name):
+        self.name = name
+
+    def __str__(self):
+        return f"<{self.name}>"
+
+    def __repr__(self):
+        return f"<{self.name}@{id(self):x}>"
+
+
+# The names HMS_TZ and hms_now were deprecated for a while and are removed in dcicutils 3.0
+# HMS_TZ = REF_TZ
+# hms_now = ref_now
